@@ -6,8 +6,8 @@ import abc
 import builtins
 
 from .. import jobs
-from ..utils import (btclients, cached_property, fs, release, signal, types,
-                     webdbs)
+from ..utils import (btclients, cached_property, configfiles, fs, release,
+                     signal, types, webdbs)
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -21,14 +21,26 @@ class TrackerConfigBase(dict):
     """
 
     _defaults = {
-        'source'     : '',
-        'exclude'    : [],
-        'add-to'     : types.Choice(
+        'source': configfiles.config_value(
             value='',
-            empty_ok=True,
-            options=(client.name for client in btclients.clients()),
+            description='Value of the "source" field in generated torrents.',
+        ),
+        'exclude': configfiles.config_value(
+            value=[],
+            description='List of regular expressions. Matching files are excluded from generated torrents.',
+        ),
+        'add_to': configfiles.config_value(
+            value=types.Choice(
+                value='',
+                empty_ok=True,
+                options=(client.name for client in btclients.clients()),
+            ),
+            description=('BitTorrent client to add torrent to after submission.'),
+        ),
+        'copy_to': configfiles.config_value(
+            value='',
+            description='Directory path to copy torrent to after submission.',
         ),
-        'copy-to'    : '',
     }
 
     defaults = {}
@@ -95,10 +107,12 @@ class TrackerJobsBase(abc.ABC):
     For a description of the arguments see the corresponding properties.
     """
 
-    def __init__(self, *, content_path, tracker, torrent_destination=None,
+    def __init__(self, *, content_path, tracker,
+                 reuse_torrent_path=None, torrent_destination=None,
                  exclude_files=(), options=None, image_host=None,
                  bittorrent_client=None, common_job_args=None):
         self._content_path = content_path
+        self._reuse_torrent_path = reuse_torrent_path
         self._tracker = tracker
         self._image_host = image_host
         self._bittorrent_client = bittorrent_client
@@ -127,6 +141,15 @@ class TrackerJobsBase(abc.ABC):
         """
         return self._tracker
 
+    @property
+    def reuse_torrent_path(self):
+        """
+        Path to an existing torrent file that matches :attr:`content_path`
+
+        See :func:`.torrent.create`.
+        """
+        return self._reuse_torrent_path
+
     @property
     def torrent_destination(self):
         """
@@ -304,6 +327,7 @@ class TrackerJobsBase(abc.ABC):
         """:class:`~.jobs.torrent.CreateTorrentJob` instance"""
         return jobs.torrent.CreateTorrentJob(
             content_path=self.content_path,
+            reuse_torrent_path=self.reuse_torrent_path,
             tracker=self.tracker,
             exclude_files=self._exclude_files,
             **self.common_job_args,
