@@ -326,25 +326,26 @@ class BbTrackerJobs(TrackerJobsBase):
             autofinish=True,
             options=(
                 {'label': 'BluRay', 'value': 'BluRay', 'regex': re.compile('BluRay')},  # [Hybrid] BluRay [Remux]
-                # {'label': 'BluRay 3D', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'BluRay RC', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'CAM', 'value': ' ', 'regex': re.compile('^ $')},
                 {'label': 'DVD5', 'value': 'DVD5', 'regex': re.compile('^DVD5$')},
                 {'label': 'DVD9', 'value': 'DVD9', 'regex': re.compile('^DVD9$')},
                 {'label': 'DVDRip', 'value': 'DVDRip', 'regex': re.compile('^DVDRip$')},
-                # {'label': 'DVDSCR', 'value': ' ', 'regex': re.compile('^ $')},
                 {'label': 'HD-DVD', 'value': 'HD-DVD', 'regex': re.compile('^HD-DVD$')},
-                # {'label': 'HDRip', 'value': ' ', 'regex': re.compile('^ $')},
                 {'label': 'HDTV', 'value': 'HDTV', 'regex': re.compile('^HDTV$')},
-                # {'label': 'PDTV', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'R5', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'SDTV', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'TC', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'TeleSync', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'VHSRip', 'value': ' ', 'regex': re.compile('^ $')},
-                # {'label': 'VODRip', 'value': ' ', 'regex': re.compile('^ $')},
                 {'label': 'WEB-DL', 'value': 'WEB-DL', 'regex': re.compile('^WEB-DL$')},
                 {'label': 'WEBRip', 'value': 'WebRip', 'regex': re.compile('^WEBRip$')},
+                # I don't know if guessit supports these
+                {'label': 'BluRay 3D', 'value': 'BluRay 3D'},
+                {'label': 'BluRay RC', 'value': 'BluRay RC'},
+                {'label': 'CAM', 'value': 'CAM'},
+                {'label': 'DVDSCR', 'value': 'DVDSCR'},
+                {'label': 'HDRip', 'value': 'HDRip'},
+                {'label': 'PDTV', 'value': 'PDTV'},
+                {'label': 'R5', 'value': 'R5'},
+                {'label': 'SDTV', 'value': 'SDTV'},
+                {'label': 'TC', 'value': 'TC'},
+                {'label': 'TeleSync', 'value': 'TeleSync'},
+                {'label': 'VHSRip', 'value': 'VHSRip'},
+                {'label': 'VODRip', 'value': 'VODRip'},
             ),
         )
 
@@ -728,7 +729,7 @@ class BbTrackerJobs(TrackerJobsBase):
     @property
     def release_info_anniversary_edition(self):
         for name in fs.file_and_parent(self.content_path):
-            match = re.search(r'(?i:[ \.](?:(\d+)th[ \.]|)Anniversary[ \.])', name)
+            match = re.search(rf'(?i:{release.DELIM}(?:(\d+)th{release.DELIM}|)Anniversary{release.DELIM})', name)
             if match:
                 if match.group(1):
                     return f'{match.group(1)}th Anniversary Edition'
@@ -1037,6 +1038,14 @@ class BbTrackerJobs(TrackerJobsBase):
                 fmt='[quote]{text}[/quote]',
             )
 
+            # Season and episode description also contains screenshots and
+            # mediainfo
+            if self.is_series_release:
+                await extend_parts(
+                    self.format_description_series_screenshots(),
+                    self.format_description_series_mediainfo(),
+                )
+
         return ''.join(parts).strip()
 
     async def format_description_summary(self):
@@ -1185,6 +1194,27 @@ class BbTrackerJobs(TrackerJobsBase):
             actors_links = [self._format_person(actor) for actor in actors]
             return ('[b]Cast[/b]: ' + ', '.join(actors_links))
 
+    async def format_description_series_screenshots(self):
+        await self.upload_screenshots_job.wait()
+        screenshot_urls = self.get_job_output(self.upload_screenshots_job)
+        screenshots_bbcode_parts = [
+            f'[img={url}]'
+            for url in screenshot_urls
+        ]
+        screenshots_bbcode = '\n\n'.join(screenshots_bbcode_parts)
+        if screenshots_bbcode:
+            return (
+                '[quote]\n'
+                f'[align=center]{screenshots_bbcode}[/align]\n'
+                '[/quote]'
+            )
+
+    async def format_description_series_mediainfo(self):
+        await self.mediainfo_job.wait()
+        mediainfo = self.get_job_output(self.mediainfo_job, slice=0)
+        if mediainfo:
+            return f'[mediainfo]{mediainfo}[/mediainfo]'
+
     # Web form data
 
     @property
@@ -1258,25 +1288,6 @@ class BbTrackerJobs(TrackerJobsBase):
         elif self.is_series_release:
             parts.append(self.get_job_output(self.series_description_job, slice=0))
 
-            # Append screenshots
-            screenshot_urls = self.get_job_output(self.upload_screenshots_job)
-            screenshots_bbcode_parts = [
-                f'[img={url}]'
-                for url in screenshot_urls
-            ]
-            screenshots_bbcode = '\n\n'.join(screenshots_bbcode_parts)
-            if screenshots_bbcode:
-                parts.append(
-                    '[quote]\n'
-                    f'[align=center]{screenshots_bbcode}[/align]\n'
-                    '[/quote]'
-                )
-
-            # Append mediainfo
-            mediainfo = self.get_job_output(self.mediainfo_job, slice=0)
-            if mediainfo:
-                parts.append(f'[mediainfo]{mediainfo}[/mediainfo]')
-
         # Append self promotion below. A closing [/quote] tag automatically
         # renders a newline. Otherwise, we must add a newline character.
         if parts and not parts[-1].endswith(('[/quote]', '\n')):
