@@ -3,6 +3,7 @@ HTTP methods with caching
 """
 
 import asyncio
+import builtins
 import collections
 import http
 import io
@@ -40,11 +41,15 @@ _client = httpx.AsyncClient(
 
 cache_directory = None
 """
-Where to store cached requests
+Where to store cached responses
 
-If this is set to a falsy value, default to :attr:`~.constants.CACHE_DIRPATH`.
+If this is set to a falsy value, default to
+:attr:`~.constants.DEFAULT_CACHE_DIRECTORY`.
 """
 
+def _get_cache_directory():
+    return cache_directory or constants.DEFAULT_CACHE_DIRECTORY
+
 
 async def close():
     """
@@ -325,7 +330,9 @@ async def _request(
             if result is not None:
                 return result
 
-        _log.debug('%s: %r: %r', method, url, params)
+        _log.debug('Sending request: %r', request)
+        # _log.debug('Request headers: %r', request.headers)
+        # _log.debug('Request data: %r', await request.aread())
         try:
             response = await _client.send(
                 request=request,
@@ -356,6 +363,8 @@ async def _request(
                 cache_file = _cache_file(method, url, params)
                 _to_cache(cache_file, response.content)
 
+            # _log.debug('Response content: %r', response.content)
+            # _log.debug('Response headers: %r', response.headers)
             return Result(
                 text=response.text,
                 bytes=response.content,
@@ -370,7 +379,7 @@ def _load_cookies(cookies):
     if isinstance(cookies, (collections.abc.Mapping, http.cookiejar.CookieJar)):
         return cookies
     elif cookies and isinstance(cookies, (str, pathlib.Path)):
-        filepath_abs = os.path.join(constants.CACHE_DIRPATH, cookies)
+        filepath_abs = os.path.join(_get_cache_directory(), cookies)
         cookie_jar = http.cookiejar.LWPCookieJar(filepath_abs)
         try:
             cookie_jar.load()
@@ -388,7 +397,7 @@ def _load_cookies(cookies):
 
 
 def _save_cookies(filepath, domain):
-    filepath_abs = os.path.join(constants.CACHE_DIRPATH, filepath)
+    filepath_abs = os.path.join(_get_cache_directory(), filepath)
     _log.debug('Saving cookies for %r to %r', domain, filepath_abs)
     file_cookie_jar = http.cookiejar.LWPCookieJar(filepath_abs)
     for cookie in _client.cookies.jar:
@@ -458,6 +467,19 @@ def _get_file_object(filepath):
 
 
 def _to_cache(cache_file, bytes):
+    if not isinstance(bytes, builtins.bytes):
+        raise TypeError(f'Not a bytes object: {bytes!r}')
+
+    # Try to remove <script> tags
+    if b'<script' in bytes:
+        try:
+            string = str(bytes, encoding='utf-8', errors='strict')
+        except UnicodeDecodeError:
+            pass
+        else:
+            string = html.purge_javascript(string)
+            bytes = string.encode('utf-8')
+
     try:
         fs.mkdir(fs.dirname(cache_file))
         with open(cache_file, 'wb') as f:
@@ -510,6 +532,6 @@ def _cache_file(method, url, params={}):
         params_str = ''
 
     return os.path.join(
-        cache_directory or constants.CACHE_DIRPATH,
+        _get_cache_directory(),
         fs.sanitize_filename(make_filename(method, url, params_str)),
     )
