@@ -32,6 +32,13 @@ logging.getLogger('rebulk').setLevel(logging.WARNING)
 _guessit = LazyModule(module='guessit.api', name='_guessit', namespace=globals())
 
 
+DELIM = r'[ \.-]'
+"""
+Regular expression that matches a single delimiter between release name
+parts, usually ``"."`` or ``" "``
+"""
+
+
 class _translated_property:
     """
     Property that is automatically translated on lookup
@@ -804,7 +811,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
         if strict:
             try:
                 scene.assert_not_abbreviated_filename(path)
-            except errors.SceneError as e:
+            except errors.SceneAbbreviatedFilenameError as e:
                 raise errors.ContentError(e)
         self._path = str(path)
         self._abspath = os.path.abspath(self._path)
@@ -857,14 +864,15 @@ class ReleaseInfo(collections.abc.MutableMapping):
         path = self._abspath
 
         # guessit doesn't detect AC3 if it's called "AC-3"
-        path = re.sub(r'([ \.])(?i:AC-?3)([ \.])', r'\1AC3\2', path)
-        path = re.sub(r'([ \.])(?i:E-?AC-?3)([ \.])', r'\1EAC3\2', path)
+        path = re.sub(rf'({DELIM})(?i:AC-?3)({DELIM})', r'\1AC3\2', path)
+        path = re.sub(rf'({DELIM})(?i:E-?AC-?3)({DELIM})', r'\1EAC3\2', path)
 
         # _log.debug('Running guessit on %r with %r', path, constants.GUESSIT_OPTIONS)
         guess = dict(_guessit.default_api.guessit(path, options=constants.GUESSIT_OPTIONS))
         # _log.debug('Original guess: %r', guess)
 
-        # We do our own episode parsing to preserve order
+        # We try to do our own episode parsing to preserve order and support
+        # multiple seasons and episodes
         for name in fs.file_and_parent(path):
             if Episodes.has_episodes_info(name):
                 guess['episodes'] = Episodes.from_string(name)
@@ -880,6 +888,23 @@ class ReleaseInfo(collections.abc.MutableMapping):
                 string.append(f'E{e:02d}')
             guess['episodes'] = Episodes.from_string(''.join(string))
 
+        # If we got an abbreviated file name (e.g. "group-titles01e02.mkv"),
+        # guessit can't (and shouldn't) handle it. Try to find episode
+        # information in it.
+        if guess['episodes']:
+            episodes_string = str(guess['episodes'])
+            if (
+                'S' in episodes_string
+                and 'E' not in episodes_string
+                and scene.is_abbreviated_filename(path)
+            ):
+                filename = fs.basename(path)
+                match = re.search(r'((?i:[SE]\d+)+)', filename)
+                if match:
+                    episodes = Episodes.from_string(match.group(1))
+                    guess['episodes'].update(episodes)
+                    _log.debug('Found episodes in abbreviated file name: %r: %r', filename, guess['episodes'])
+
         return guess
 
     @property
@@ -887,20 +912,9 @@ class ReleaseInfo(collections.abc.MutableMapping):
         return _guessit.default_api.advanced_config
 
     def _get_type(self):
-        for name in fs.file_and_parent(self._path):
-            if Episodes.has_episodes_info(name):
-                episodes = Episodes.from_string(name)
-                if any(episodes.values()):
-                    return ReleaseType.episode
-                else:
-                    return ReleaseType.season
-
-        guessit_type = _as_string(self._guess.get('type'))
-        if not guessit_type:
-            return ReleaseType.unknown
-        elif guessit_type == 'episode':
-            # guessit doesn't detect season packs; check if at least one episode
-            # is specified
+        # guessit doesn't differentiate between episodes and season packs.
+        # Check if at least one episode is specified.
+        if self['episodes']:
             if any(episodes for episodes in self['episodes'].values()):
                 return ReleaseType.episode
             else:
@@ -909,11 +923,11 @@ class ReleaseInfo(collections.abc.MutableMapping):
             return ReleaseType.movie
 
     _title_split_regex = re.compile(
-        r'[ \.]+(?:'
+        r'[ \.](?:'
         r'\d{4}|'  # Year
-        r'(?i:[SE]\d+)+|'
+        r'(?i:[SE]\d+)+|'  # Sxx or SxxExx
         r'((?i:Season|Episode)[ \.]*\d+[ \.]*)+|'
-        r')[ \.]+'
+        r')[ \.]'
     )
 
     @cached_property
@@ -933,7 +947,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
                 return self._title_split_regex.split(name, maxsplit=1)[-1]
 
         # Default to the file/parent that contains either " " or "." (without
-        # file extension)
+        # the "." from the extension)
         for name in fs.file_and_parent(path_no_ext):
             if ' ' in name or '.' in name:
                 return name
@@ -941,7 +955,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
         # Default to file name
         return fs.basename(path_no_ext)
 
-    _title_aka_regex = re.compile(r'[ \.]+AKA[ \.]+')
+    _title_aka_regex = re.compile(rf'{DELIM}AKA{DELIM}')
 
     @cached_property
     def _title_parts(self):
@@ -1012,14 +1026,14 @@ class ReleaseInfo(collections.abc.MutableMapping):
         'Theatrical Cut': re.compile(r'Theatrical'),
         'Ultimate Cut': re.compile(r'Ultimate'),
     }
-    _proper_repack_regex = re.compile(r'(?:[ \.]|^)((?i:proper|repack\d*))(?:[ \.]|$)')
+    _proper_repack_regex = re.compile(rf'(?:{DELIM}|^)((?i:proper|repack\d*))(?:{DELIM}|$)')
     _hdr_regexes = {
-        'Dolby Vision': re.compile(r'(?:[ \.]|^)(?i:DV|DoVi|Dolby[ \.]Vision)(?:[ \.]|$)'),
-        'HDR10+': re.compile(r'(?:[ \.]|^)(?i:HDR10\+)(?:[ \.]|$)'),
-        'HDR10': re.compile(r'(?:[ \.]|^)(?i:HDR10)(?:[^\+]|$)'),
-        'HDR': re.compile(r'(?:[ \.]|^)(?i:HDR)(?:[^10\+]|$)'),
+        'Dolby Vision': re.compile(rf'(?:{DELIM}|^)(?i:DV|DoVi|Dolby{DELIM}Vision)(?:{DELIM}|$)'),
+        'HDR10+': re.compile(rf'(?:{DELIM}|^)(?i:HDR10\+)(?:{DELIM}|$)'),
+        'HDR10': re.compile(rf'(?:{DELIM}|^)(?i:HDR10)(?:[^\+]|$)'),
+        'HDR': re.compile(rf'(?:{DELIM}|^)(?i:HDR)(?:[^10\+]|$)'),
     }
-    _remastered_regex = re.compile(r'(?:[ \.]|^)((?i:4k[ \.]+|)(?i:remaster(?:ed|)|restored))(?:[ \.]|$)')
+    _remastered_regex = re.compile(rf'(?:{DELIM}|^)((?i:4k{DELIM}+|)(?i:remaster(?:ed|)|restored))(?:{DELIM}|$)')
 
     def _get_edition(self):
         edition = _as_list(self._guess.get('edition'))
@@ -1067,7 +1081,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def _get_resolution(self):
         return _as_string(self._guess.get('screen_size', ''))
 
-    _streaming_service_regex = re.compile(r'[ \.]([A-Z]+)[ \.](?i:WEB-?(?:DL|Rip))(?:[ \.]|$)')
+    _streaming_service_regex = re.compile(rf'{DELIM}([A-Z]+){DELIM}(?i:WEB-?(?:DL|Rip))(?:{DELIM}|$)')
     _streaming_service_translation = {
         re.compile(r'(?i:IT)'): 'iT',
         re.compile(r'(?i:ATVP)'): 'APTV',
@@ -1108,13 +1122,14 @@ class ReleaseInfo(collections.abc.MutableMapping):
     _source_translation = {
         re.compile(r'(?i:blu-?ray)') : 'BluRay',
         re.compile(r'(?i:dvd-?rip)') : 'DVDRip',
+        re.compile(r'(?i:tv-?rip)')  : 'TVRip',
         re.compile(r'(?i:web-?dl)')  : 'WEB-DL',
         re.compile(r'(?i:web-?rip)') : 'WEBRip',
         re.compile(r'(?i:web)')      : 'WEB-DL',
     }
     # Look for "Hybrid" after year or season
-    _hybrid_regex = re.compile(r'[ \.]hybrid[ \.]', flags=re.IGNORECASE)
-    _web_source_regex = re.compile(r'[ \.](WEB-?(?:DL|Rip))(?:[ \.]|$)', flags=re.IGNORECASE)
+    _hybrid_regex = re.compile(rf'{DELIM}hybrid{DELIM}', flags=re.IGNORECASE)
+    _web_source_regex = re.compile(rf'{DELIM}(WEB-?(?:DL|Rip))(?:{DELIM}|$)', flags=re.IGNORECASE)
 
     def _get_source(self):
         source = self._guess.get('source', '')
@@ -1133,16 +1148,18 @@ class ReleaseInfo(collections.abc.MutableMapping):
                 source = match.group(1)
 
         elif source == 'DVD':
-            # Detect DVDRip
             if 'Rip' in self._guess.get('other', ()):
                 source = 'DVDRip'
-            # Detect DVD images
             else:
                 if 'DVD9' in self.release_name_params:
                     source = 'DVD9'
                 elif 'DVD5' in self.release_name_params:
                     source = 'DVD5'
 
+        elif source == 'TV':
+            if 'Rip' in self._guess.get('other', ()):
+                source = 'TVRip'
+
         if source:
             # Fix spelling
             for regex,source_fixed in self._source_translation.items():
@@ -1201,7 +1218,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
 
             return audio_codec
 
-    _audio_channels_regex = re.compile(r'[ \.](\d\.\d)[ \.]')
+    _audio_channels_regex = re.compile(rf'{DELIM}(\d\.\d){DELIM}')
 
     def _get_audio_channels(self):
         audio_channels = _as_string(self._guess.get('audio_channels', ''))
@@ -1211,8 +1228,8 @@ class ReleaseInfo(collections.abc.MutableMapping):
                 return match.group(1)
         return audio_channels
 
-    _x264_regex = re.compile(r'(?:^|[\. ])(?i:x264)(?:[\. -]|$)')
-    _x265_regex = re.compile(r'(?:^|[\. ])(?i:x265)(?:[\. -]|$)')
+    _x264_regex = re.compile(rf'(?:{DELIM}|^)(?i:x264)(?:{DELIM}|$)')
+    _x265_regex = re.compile(rf'(?:{DELIM}|^)(?i:x265)(?:{DELIM}|$)')
 
     def _get_video_codec(self):
         video_codec = _as_string(self._guess.get('video_codec', ''))
@@ -1229,7 +1246,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def _get_group(self):
         return _as_string(self._guess.get('release_group', ''))
 
-    _has_commentary_regex = re.compile(r'[\. ](?i:plus[\. -]+comm|commentary)[\. -]')
+    _has_commentary_regex = re.compile(rf'{DELIM}(?i:plus{DELIM}+comm|commentary){DELIM}')
 
     def _get_has_commentary(self):
         if self._guess.get('has_commentary', None) is None:
@@ -1246,15 +1263,25 @@ class ReleaseInfo(collections.abc.MutableMapping):
 
 class Episodes(dict):
     """
-    :class:`dict` subclass that maps season numbers to sequences of episode
-    numbers
+    :class:`dict` subclass that maps season numbers to lists of episode numbers
 
     All keys and values are :class:`str` objects. All episodes from a season are
     indicated by an empty sequence. For episodes from any sason, the key is an
     empty string.
+
+    This class accepts the same arguments as :class:`dict`.
+
+    To provide seasons as keyword arguments, you need to prefix "S" to each
+    keyword. This is because numbers can't be keyword arguments, but it also
+    looks nicer.
+
+    >>> e = Episodes({"1": ["1", "2", "3"], "2": []})
+    >>> e.update(S01=[3, "4"], S3=range(2, 4), s05=[], S=[10, "E11", 12])
+    >>> e
+    >>> Episodes({'1': ['1', '2', '3', '4'], '2': [], '3': ['2', '3'], '5': [], '': ['10', '11', '12']})
     """
 
-    regex = re.compile(r'(?:^|[\. ])((?i:[SE]\d+)+)(?:[\. ]|$)')
+    regex = re.compile(rf'(?:{DELIM}|^)((?i:[SE]\d+)+)(?:{DELIM}|$)')
     """Regular expression that matches "S01E02"-like episode information"""
 
     @classmethod
@@ -1262,6 +1289,13 @@ class Episodes(dict):
         """Whether `string` contains "S01E02"-like episode information"""
         return bool(cls.regex.search(string))
 
+    _is_episodes_info_regex = re.compile(r'^(?i:[SE]\d+)+$')
+
+    @classmethod
+    def is_episodes_info(cls, string):
+        """Whether `string` is "S01E02"-like episode information and nothing else"""
+        return bool(cls._is_episodes_info_regex.search(string))
+
     @classmethod
     def from_string(cls, value):
         """
@@ -1282,12 +1316,14 @@ class Episodes(dict):
             >>> Episodes.from_string('E01S03E06.bar.E02')
             {'': ('1', '2',), '3': ('6',)}
         """
-        seasons = collections.defaultdict(lambda: set())
+        def only_digits(string):
+            return ''.join(c for c in string if c in '0123456789')
 
         def split_episodes(string):
-            return {str(int(e)) for e in string.split('E') if e.strip()}
+            return {only_digits(e) for e in string.split('E') if e.strip()}
 
-        for word in re.split(r'[ \.]', str(value)):
+        seasons = collections.defaultdict(lambda: set())
+        for word in re.split(r'[\. ]+', str(value)):
             word = word.upper()
             if cls.has_episodes_info(word):
                 for part in (k for k in word.split('S') if k):
@@ -1296,6 +1332,7 @@ class Episodes(dict):
                     episodes = re.sub(r'^\d+', '', part)
                     episodes = split_episodes(episodes) if episodes else ()
                     seasons[season].update(episodes)
+
         args = {season: tuple(natsort.natsorted(episodes))
                 for season, episodes in natsort.natsorted(seasons.items())}
         return cls(args)
@@ -1323,29 +1360,79 @@ class Episodes(dict):
         return episodes
 
     def __init__(self, *args, **kwargs):
-        def number(name, value):
-            if isinstance(value, int):
-                if value >= 0:
-                    return str(value)
-                else:
-                    raise ValueError(f'Invalid {name}: {value!r}')
-            elif isinstance(value, str):
-                if value == '' or value.isdigit():
-                    return str(value)
-                else:
-                    raise ValueError(f'Invalid {name}: {value!r}')
-            else:
-                raise TypeError(f'Invalid {name}: {value!r}')
+        super().__init__()
+        self._set(*args, **kwargs)
 
-        validated_args = {}
-        for season, episodes in dict(*args, **kwargs).items():
-            season = number('season', season)
-            if not isinstance(episodes, collections.abc.Iterable):
-                raise TypeError(f'Invalid episodes: {episodes!r}')
+    def update(self, *args, clear=False, **kwargs):
+        """
+        Set specific episodes from specific seasons, remove all other episodes and
+        seasons
+
+        :params bool clear: Whether to remove all seasons and episodes first
+        """
+        if clear:
+            self.clear()
+        self._set(*args, **kwargs)
+
+    def _set(self, *args, **kwargs):
+        # Validate all values before applying any changes
+        validated = {}
+        update = dict(*args, **kwargs)
+        for season, episodes in update.items():
+            season = self._normalize_season(season)
+            if not isinstance(episodes, collections.abc.Iterable) or isinstance(episodes, str):
+                episodes = [self._normalize_episode(episodes)]
             else:
-                episodes = tuple(number('episode', e) for e in episodes)
-            validated_args[season] = episodes
-        return super().__init__(validated_args)
+                episodes = [self._normalize_episode(e) for e in episodes]
+
+            if season in validated:
+                validated[season].extend(episodes)
+            else:
+                validated[season] = episodes
+
+        # Set validated values
+        for season, episodes in validated.items():
+            if season in self:
+                self[season].extend(episodes)
+            else:
+                self[season] = episodes
+
+            # Remove duplicates
+            self[season][:] = set(self[season])
+
+            # Sort naturally
+            self[season].sort(key=natsort.natsort_key)
+
+    def _normalize_season(self, value):
+        return self._normalize(value, name='season', prefix='S', empty_string_ok=True)
+
+    def _normalize_episode(self, value):
+        return self._normalize(value, name='episode', prefix='E', empty_string_ok=False)
+
+    def _normalize(self, value, name, prefix=None, empty_string_ok=False):
+        if isinstance(value, int):
+            if value >= 0:
+                return str(value)
+
+        elif isinstance(value, str):
+            if value == '' and empty_string_ok:
+                return str(value)
+
+            if value.isdigit():
+                return str(int(value))
+
+            if prefix and len(value) >= len(prefix):
+                prefix_ = value[:len(prefix)].casefold()
+                if prefix_ == prefix.casefold():
+                    actual_value = value[len(prefix):]
+                    return self._normalize(
+                        actual_value,
+                        name=name,
+                        prefix=None,
+                        empty_string_ok=empty_string_ok,
+                    )
+
+        raise TypeError(f'Invalid {name}: {value!r}')
 
     def remove_specific_episodes(self):
         """Remove episodes from each season, leaving only complete seasons"""
