@@ -2,11 +2,12 @@
 Search for scene release
 """
 
+import collections
 import re
 
-from ... import errors
-from .. import LazyModule, release
-from . import common
+from ... import constants, errors
+from .. import LazyModule, asyncmemoize, fs, release
+from . import common, verify
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -14,7 +15,8 @@ _log = logging.getLogger(__name__)
 natsort = LazyModule(module='natsort', namespace=globals())
 
 
-async def search(*args, dbs=('predb', 'srrdb'), **kwargs):
+@asyncmemoize
+async def search(query, dbs=('predbovh', 'srrdb'), only_existing_releases=None):
     """
     Search scene databases
 
@@ -24,27 +26,53 @@ async def search(*args, dbs=('predb', 'srrdb'), **kwargs):
     Failed requests are ignored unless all requests fail, in which case they are
     combined into a single :class:`~.errors.RequestError`.
 
+    If there are no results and `query` is a directory path that looks like a
+    season pack, perform one search per video file in that directory or any
+    subdirectory. This is necessary to find mixed season packs.
+
+    :param query: :class:`SceneQuery` object or :class:`str` to pass to
+        :meth:`SceneQuery.from_string` or :class:`~.collections.abc.Mapping` to
+        pass to :meth:`SceneQuery.from_release`
     :param dbs: Sequence of :attr:`~.base.SceneDbApiBase.name` values
+    :param only_existing_releases: See :meth:`.SceneQuery.search`
 
-    All other arguments are forwarded to :attr:`~.base.SceneDbApiBase.search`.
-
-    :return: Sequence of release names as :class:`str`
+    :return: Sequence of release names (:class:`str`)
 
     :raise RequestError: if all search requests fail
     """
+    path = None
+    if isinstance(query, str):
+        path = query
+        query = SceneQuery.from_string(query)
+    elif isinstance(query, collections.abc.Mapping):
+        query = SceneQuery.from_release(query)
+
+    results = await _multisearch(dbs, query, only_existing_releases)
+    if results:
+        return results
+    elif path:
+        combined_results = []
+        for episode_query in _generate_episode_queries(path):
+            results = await _multisearch(dbs, episode_query, only_existing_releases)
+            combined_results.extend(results)
+        return combined_results
+
+
+async def _multisearch(dbs, query, only_existing_releases):
+    # Send the same query to multiple DBs
     from . import scenedb
 
     exceptions = []
     for db_name in dbs:
         db = scenedb(db_name)
         try:
-            result = await db.search(*args, **kwargs)
+            results = await db.search(query, only_existing_releases=only_existing_releases)
         except errors.RequestError as e:
             _log.debug('Collecting scene search error: %r', e)
             exceptions.append(e)
         else:
-            _log.debug('Returning first scene search result: %r', result)
-            return result
+            _log.debug('Returning scene search results: %r', results)
+            return results
 
     if exceptions:
         msg = 'All queries failed: ' + ', '.join(str(e) for e in exceptions)
@@ -53,6 +81,17 @@ async def search(*args, dbs=('predb', 'srrdb'), **kwargs):
         return []
 
 
+def _generate_episode_queries(path):
+    info = release.ReleaseInfo(path)
+    if info['type'] is release.ReleaseType.season:
+        # Create `query` from each `episode_path`
+        episode_paths = fs.file_list(path, extensions=constants.VIDEO_FILE_EXTENSIONS)
+        for episode_path in episode_paths:
+            _log.debug('Creating query for episode: %r', episode_path)
+            if not verify.is_abbreviated_filename(episode_path):
+                yield SceneQuery.from_string(episode_path)
+
+
 class SceneQuery:
     """
     Query for scene release databases
@@ -62,14 +101,6 @@ class SceneQuery:
     :param episodes: :class:`~.release.Episodes`-like mapping
     """
 
-    def __init__(self, *keywords, group='', episodes={}):
-        # Split each keyword
-        self._keywords = tuple(k.strip() for kw in keywords
-                               for k in str(kw).split()
-                               if k.strip() and k.strip() != '-')
-        self._group = str(group)
-        self._episodes = episodes
-
     @classmethod
     def from_string(cls, string):
         """
@@ -84,7 +115,7 @@ class SceneQuery:
             :class:`~.errors.ContentError`
         """
         try:
-            return cls.from_release(release.ReleaseInfo(string, strict=True))
+            return cls.from_release(release.ReleaseInfo(string, strict=False))
         except errors.ContentError as e:
             raise errors.SceneError(e)
 
@@ -99,17 +130,27 @@ class SceneQuery:
             ``resolution``, ``source``, ``video_codec`` and ``group``.
         """
         # Replace H.264/5 with H264/5
-        release['video_codec'] = re.sub(r'\.', '', release['video_codec'])
+        release['video_codec'] = re.sub(r'\.', '', release.get('video_codec', ''))
 
         # Replace WEB-DL with WEB
         if release.get('source') == 'WEB-DL':
             release['source'] = 'WEB'
 
-        # Group and episodes are handled separately the other keywords
+        # Group and episodes are handled separately from the other keywords
         needed_keys = [k for k in common.get_needed_keys(release)
                        if k not in ('group', 'episodes')]
         keywords = [release[key] for key in needed_keys
                     if key in release]
+
+        # Add single episode ("SxxEyy"), but not season pack ("Sxx") or anything
+        # else (e.g. "SxxEyyEzz")
+        seasons = tuple(release.get('episodes', {}).keys())
+        episodes = tuple(episode
+                         for episodes in release.get('episodes', {}).values()
+                         for episode in episodes)
+        if len(seasons) == 1 and len(episodes) == 1:
+            keywords.append(f'S{int(seasons[0]):02d}E{int(episodes[0]):02d}')
+
         query = cls(
             *keywords,
             group=release.get('group', ''),
@@ -118,7 +159,38 @@ class SceneQuery:
         _log.debug('SceneQuery from %r: %r', release, query)
         return query
 
-    async def search(self, search_coro_func, only_existing_releases=True):
+    def __init__(self, *keywords, group='', episodes={}):
+        # Split each keyword at spaces
+        kws = (k.strip()
+               for kw in keywords
+               for k in str(kw).split())
+
+        # Remove season packs (keep "SxxEyy" but not "Sxx") because scene
+        # releases are not season packs and we don't find anything if we look
+        # for "S05". For season packs, we try to find all episodes of all
+        # seasons and extract the relevant episodes in _handle_results().
+        def exclude_season_pack(kw):
+            if release.Episodes.is_episodes_info(kw):
+                episodes = release.Episodes.from_string(kw)
+                for season in tuple(episodes):
+                    if not episodes[season]:
+                        del episodes[season]
+                return str(episodes)
+            else:
+                return kw
+
+        kws = (exclude_season_pack(kw) for kw in kws)
+
+        # Remove empty keywords
+        # (I forgot why "-" is removed. Please explain if you know!)
+        kws = (kw for kw in kws
+               if kw and kw != '-')
+
+        self._keywords = tuple(kws)
+        self._group = str(group) if group else None
+        self._episodes = episodes
+
+    async def search(self, search_coro_func, only_existing_releases=None):
         """
         Pre-process query and post-process results from `search_coro_func`
 
@@ -127,22 +199,17 @@ class SceneQuery:
 
         :param search_coro_func: Coroutine function with the call signature
             `(keywords, group)` that returns a sequence of release names
-        :param bool only_existing_releases: If this is truthy, the results
-            contain all episodes for every season pack in :attr:`keywords`.
-            Otherwise, fake season pack release names are included in the
-            returned results.
-
-        Any `keywords` that look like "S01", "S02E03", etc are removed in the
-        search request. The search results are then filtered for the specific
-        :attr:`episodes` given and returned in natural sort order.
+        :param bool only_existing_releases: If this is truthy (the default), the
+            results contain all episodes for every season pack in
+            :attr:`keywords`.  Otherwise, fake season pack release names are
+            included in the returned results.
         """
-        keywords = tuple(kw for kw in self.keywords
-                         if not re.match(r'^(?i:[SE]\d+|)+$', kw))
-        _log.debug('Searching for scene release: %r', keywords)
-        results = await search_coro_func(keywords, group=self.group)
+        if only_existing_releases is None:
+            only_existing_releases = True
+        results = await search_coro_func(self.keywords, group=self.group)
         return self._handle_results(results, only_existing_releases)
 
-    def _handle_results(self, results, only_existing_releases=True):
+    def _handle_results(self, results, only_existing_releases):
         def sorted_and_deduped(results):
             return natsort.natsorted(set(results), key=str.casefold)
 
@@ -153,12 +220,13 @@ class SceneQuery:
             _log.debug('Looking for existing episodes: %r', self.episodes)
 
             def get_wanted_episodes(season):
-                # Combine episodes from any season ('') with episodes from given season
+                # Combine episodes from any season with episodes from given
+                # season (season being empty string means "any season")
                 eps = None
                 if '' in self.episodes:
                     eps = self.episodes['']
                 if season in self.episodes:
-                    eps = (eps or ()) + self.episodes[season]
+                    eps = (eps or []) + self.episodes[season]
                 return eps
 
             # Translate single episodes into season packs.
@@ -166,16 +234,18 @@ class SceneQuery:
             for result in results:
                 for result_season, result_eps in release.Episodes.from_string(result).items():
                     wanted_episodes = get_wanted_episodes(result_season)
-                    # () means season pack
-                    if wanted_episodes == ():
+
+                    # [] means season pack
+                    if wanted_episodes == []:
                         if only_existing_releases:
                             # Add episode from wanted season pack
                             _log.debug('Adding episode from season pack: %r', result)
                             matches.append(result)
                         else:
-                            season_pack = self._create_season_pack_name(result)
+                            season_pack = common.get_season_pack_name(result)
                             _log.debug('Adding season pack: %r', season_pack)
                             matches.append(season_pack)
+
                     elif wanted_episodes is not None:
                         for ep in result_eps:
                             if ep in wanted_episodes:
@@ -184,24 +254,6 @@ class SceneQuery:
 
             return sorted_and_deduped(matches)
 
-    @staticmethod
-    def _create_season_pack_name(release_name):
-        # Remove episode from release name to create season pack name
-        season_pack = re.sub(r'\b(S\d{2,})E\d{2,}\b', r'\1', release_name)
-
-        # Remove episode title
-        release_info = release.ReleaseInfo(release_name)
-        if release_info['episode_title']:
-            # Escape special characters in each word and
-            # join words with "space or period" regex
-            episode_title_regex = r'[ \.]{1,}'.join(
-                re.escape(word)
-                for word in release_info['episode_title'].split()
-            )
-            season_pack = re.sub(rf'\b{episode_title_regex}\W', '', season_pack)
-
-        return season_pack
-
     @property
     def keywords(self):
         """Sequence of search terms"""
