@@ -17,7 +17,7 @@ torf = LazyModule(module='torf', namespace=globals())
 
 
 def create(*, content_path, announce, source, torrent_path,
-           init_callback, progress_callback,
+           init_callback, progress_callback, info_callback,
            overwrite=False, exclude=(), reuse_torrent_path=None):
     """
     Generate and write torrent file
@@ -28,10 +28,10 @@ def create(*, content_path, announce, source, torrent_path,
         the torrent unique for each tracker to avoid cross-seeding issues, so it
         is usually the tracker's abbreviated name.
     :param str torrent_path: Path of the generated torrent file
-    :param str init_callback: Callable that is called once before torrent
-        generation commences. It gets `content_path` as a tree where each node
-        is a tuple in which the first item is the directory name and the second
-        item is a sequence of `(file_name, file_size)` tuples.
+    :param init_callback: Callable that is called once before torrent generation
+        commences. It gets `content_path` as a tree where each node is a tuple
+        in which the first item is the directory name and the second item is a
+        sequence of `(file_name, file_size)` tuples.
 
         Example:
 
@@ -52,17 +52,25 @@ def create(*, content_path, announce, source, torrent_path,
                    ('Yah.mkv', 3883247384),
                )),
            )
-    :param str progress_callback: Callable that is called at regular intervals
-        with a :class:`CreateTorrentStatus` object as a positional argument.
-        Torrent creation is cancelled if `progress_callback` returns `True` or
-        any other truthy value.
+    :param progress_callback: Callable that is called at regular intervals with
+        a :class:`CreateTorrentProgress` object as a positional argument
+    :param info_callback: Callable that is called with an informational message
     :param bool overwrite: Whether to overwrite `torrent_path` if it exists
     :param exclude: Sequence of regular expressions that are matched against
         file system paths. Matching files are not included in the torrent.
     :param reuse_torrent_path: Path to existing torrent file to get hashed
-        pieces and piece size from. This argument is ignored if the files in the
-        provided torrent do not match the files we want or if it can't be read
-        for any reason.
+        pieces and piece size from.
+
+        If this is a directory, search it recursively for ``*.torrent`` files
+        and use the first one (in natural sort order) that matches.
+
+        Non-existing or otherwise unreadable paths as well as falsy values
+        (e.g. ``""`` or `None`) are silently ignored.
+
+        If this is a sequence, its items are handled as described above.
+
+    Callbacks can cancel the torrent creation by returning `True` or any other
+    truthy value.
 
     :raise TorrentError: if anything goes wrong
 
@@ -73,24 +81,23 @@ def create(*, content_path, announce, source, torrent_path,
     if not source:
         raise errors.TorrentError('Source is empty')
 
+    torrent = None
     if overwrite or not _path_exists(torrent_path):
-        torrent = None
-
-        # Try to read piece hashes from cache
-        torrent = _read_cache_torrent(
+        # Try to get existing torrent from global cache or `reuse_torrent_path`
+        torrent = _get_cached_torrent(
             content_path=content_path,
-            cache_torrent_path=reuse_torrent_path,
             exclude=exclude,
+            metadata={
+                'trackers': (announce,),
+                'source': source,
+            },
+            reuse_torrent_path=reuse_torrent_path,
+            info_callback=info_callback,
         )
-        if torrent:
-            # Update metadata in generic torrent
-            torrent.trackers = (announce,)
-            torrent.source = source
-            if reuse_torrent_path:
-                _store_cache_torrent(torrent)
-        else:
-            # Reading cached piece hashes failed for some reason
-            torrent = _generate_torrent(
+
+        if not torrent:
+            # Create pieces hashes
+            torrent = _get_generated_torrent(
                 content_path=content_path,
                 announce=announce,
                 source=source,
@@ -98,19 +105,65 @@ def create(*, content_path, announce, source, torrent_path,
                 progress_callback=progress_callback,
                 init_callback=init_callback,
             )
-            if torrent.is_ready:
-                _store_cache_torrent(torrent)
 
-        # Write torrent file
+    if torrent and torrent.is_ready:
+        # Write generic torrent
+        _store_generic_torrent(torrent)
+
+        # Write torrent to `torrent_path`
         try:
             torrent.write(torrent_path, overwrite=True)
         except torf.TorfError as e:
             raise errors.TorrentError(e)
-
-    return torrent_path
+        else:
+            return torrent_path
 
 
-def _generate_torrent(*, content_path, announce, source, exclude, init_callback, progress_callback):
+def _get_cached_torrent(content_path, exclude, metadata, reuse_torrent_path, info_callback):
+    def with_updated_metadata(torrent):
+        for name, value in metadata.items():
+            setattr(torrent, name, value)
+        return torrent
+
+    if not reuse_torrent_path:
+        reuse_torrent_paths = []
+    elif isinstance(reuse_torrent_path, str):
+        reuse_torrent_paths = [reuse_torrent_path]
+    elif isinstance(reuse_torrent_path, collections.abc.Iterable):
+        reuse_torrent_paths = [p for p in reuse_torrent_path if p]
+    else:
+        raise ValueError(f'Invalid reuse_torrent_path value: {reuse_torrent_path!r}')
+
+    # Try generic torrent first
+    try:
+        torrent = torf.Torrent(path=content_path, exclude_regexs=exclude)
+    except torf.TorfError:
+        pass
+    else:
+        reuse_torrent_paths.insert(0, _get_generic_torrent_path(
+            torrent=torrent,
+            create_directory=False,
+        ))
+
+    # Iterate over .torrent files in each path in `reuse_torrent_paths`.
+    # If `reuse_torrent_path` is not a directory, it's iterated over by
+    # fs.file_list().
+    for path in reuse_torrent_paths:
+        for torrent_path in fs.file_list(path, extensions=('torrent',)):
+            cancelled = info_callback(torrent_path)
+            if cancelled:
+                return None
+            else:
+                torrent = _read_torrent(
+                    content_path=content_path,
+                    exclude=exclude,
+                    torrent_path=torrent_path,
+                )
+                if torrent:
+                    return with_updated_metadata(torrent)
+
+
+def _get_generated_torrent(*, content_path, announce, source, exclude, init_callback, progress_callback):
     callback = _CreateTorrentCallbackWrapper(progress_callback)
     try:
         torrent = torf.Torrent(
@@ -122,52 +175,53 @@ def _generate_torrent(*, content_path, announce, source, exclude, init_callback,
             created_by=f'{__project_name__} {__version__}',
             creation_date=time.time(),
         )
-        init_callback(_make_file_tree(torrent.filetree))
-        torrent.generate(callback=callback, interval=1.0)
+        cancelled = init_callback(_make_file_tree(torrent.filetree))
+        if cancelled:
+            return None
+        else:
+            torrent.generate(callback=callback, interval=1.0)
     except torf.TorfError as e:
         raise errors.TorrentError(e)
     else:
         return torrent
 
 
-def _store_cache_torrent(torrent):
-    cache_torrent = torf.Torrent(
+def _store_generic_torrent(torrent):
+    generic_torrent = torf.Torrent(
         private=True,
         created_by=f'{__project_name__} {__version__}',
         creation_date=time.time(),
         comment='This torrent is used to cache previously hashed pieces.',
     )
-    _copy_torrent_info(torrent, cache_torrent)
-    cache_torrent_path = _get_cache_torrent_path(cache_torrent, create_directory=True)
-    cache_torrent.write(cache_torrent_path, overwrite=True)
+    _copy_torrent_info(torrent, generic_torrent)
+    generic_torrent_path = _get_generic_torrent_path(generic_torrent, create_directory=True)
+    generic_torrent.write(generic_torrent_path, overwrite=True)
 
 
-def _read_cache_torrent(content_path, exclude, cache_torrent_path=None):
+def _read_torrent(content_path, exclude, torrent_path):
     try:
-        # Create the Torrent we're going to use
-        torrent = torf.Torrent(
+        # Create the Torrent we're going to use, but it's missing the hashes
+        new_torrent = torf.Torrent(
             path=content_path,
             exclude_regexs=exclude,
             private=True,
             created_by=f'{__project_name__} {__version__}',
             creation_date=time.time(),
         )
-        # Create the Torrent to get hashed pieces from
-        if not cache_torrent_path:
-            cache_torrent_path = _get_cache_torrent_path(torrent, create_directory=False)
-        cache_torrent = torf.Torrent.read(cache_torrent_path)
+        # Read existing torrent
+        old_torrent = torf.Torrent.read(torrent_path)
     except torf.TorfError:
         pass
     else:
-        # Make sure the cached torrent metainfo matches what we expect
-        id_wanted = _get_torrent_id(torrent)
-        id_cached = _get_torrent_id(cache_torrent)
-        if id_cached == id_wanted:
-            _copy_torrent_info(cache_torrent, torrent)
-            return torrent
+        # If both torrents match, we can copy the hashes
+        id_new = _get_torrent_id(new_torrent)
+        id_old = _get_torrent_id(old_torrent)
+        if id_old == id_new:
+            _copy_torrent_info(old_torrent, new_torrent)
+            return new_torrent
 
 
-def _get_cache_torrent_path(torrent, create_directory=True):
+def _get_generic_torrent_path(torrent, create_directory=True):
     directory = constants.GENERIC_TORRENTS_DIRPATH
     if create_directory:
         try:
@@ -214,22 +268,22 @@ class _CreateTorrentCallbackWrapper:
     def __init__(self, callback):
         self._callback = callback
         self._progress_samples = []
-        self._time_started = datetime.datetime.now()
+        self._time_started = time.time()
 
     def __call__(self, torrent, filepath, pieces_done, pieces_total):
-        time_now = datetime.datetime.now()
+        time_now = time.time()
         percent_done = pieces_done / pieces_total * 100
         seconds_elapsed = time_now - self._time_started
 
-        def progress_samples_trim_condition(samples):
-            oldest_sample_age = time_now - samples[0][0]
-            return oldest_sample_age > datetime.timedelta(seconds=10)
+        def get_sample_age(sample):
+            time_sample = sample[0]
+            return time_now - time_sample
 
-        self._maintain_samples(
-            samples=self._progress_samples,
-            new_sample=(time_now, pieces_done),
-            trim_condition=progress_samples_trim_condition,
-        )
+        # Limit number of samples to use for estimates
+        samples = self._progress_samples
+        samples.append((time_now, pieces_done))
+        while samples and get_sample_age(samples[0]) > 10:
+            del samples[0]
 
         # Estimate how long torrent creation will take
         if len(self._progress_samples) >= 2:
@@ -242,34 +296,29 @@ class _CreateTorrentCallbackWrapper:
             bytes_per_second = types.Bytes(pieces_per_second * torrent.piece_size)
             pieces_remaining = pieces_total - pieces_done
             bytes_remaining = pieces_remaining * torrent.piece_size
-            seconds_remaining = datetime.timedelta(seconds=bytes_remaining / bytes_per_second)
+            seconds_remaining = bytes_remaining / bytes_per_second
         else:
             bytes_per_second = types.Bytes(0)
-            seconds_remaining = datetime.timedelta(seconds=0)
+            seconds_remaining = 0
 
         seconds_total = seconds_elapsed + seconds_remaining
         time_finished = self._time_started + seconds_total
 
-        status = CreateTorrentStatus(
+        progress = CreateTorrentProgress(
             bytes_per_second=bytes_per_second,
             filepath=filepath,
             percent_done=percent_done,
             piece_size=torrent.piece_size,
             pieces_done=pieces_done,
             pieces_total=pieces_total,
-            seconds_elapsed=seconds_elapsed,
-            seconds_remaining=seconds_remaining,
-            seconds_total=seconds_total,
-            time_finished=time_finished,
-            time_started=self._time_started,
+            seconds_elapsed=datetime.timedelta(seconds=seconds_elapsed),
+            seconds_remaining=datetime.timedelta(seconds=seconds_remaining),
+            seconds_total=datetime.timedelta(seconds=seconds_total),
+            time_finished=datetime.datetime.fromtimestamp(time_finished),
+            time_started=datetime.datetime.fromtimestamp(self._time_started),
             total_size=torrent.size,
         )
-        return self._callback(status)
-
-    def _maintain_samples(self, new_sample, samples, trim_condition):
-        samples.append(new_sample)
-        while samples and trim_condition(samples):
-            del samples[0]
+        return self._callback(progress)
 
     def _get_average(self, samples, weight_factor, get_value=lambda sample: sample):
         # Give recent samples more weight than older samples
@@ -288,8 +337,8 @@ class _CreateTorrentCallbackWrapper:
         ) / sum(weights)
 
 
-class CreateTorrentStatus(collections.namedtuple(
-    typename='CreateTorrentStatus',
+class CreateTorrentProgress(collections.namedtuple(
+    typename='CreateTorrentProgress',
     field_names=(
         'bytes_per_second',
         'filepath',
