@@ -6,6 +6,8 @@ import functools
 import re
 import string
 
+import unidecode
+
 from ... import utils
 from ..types import ReleaseType
 from . import common
@@ -205,7 +207,7 @@ class ImdbApi(WebDbApiBase):
         re.compile(r'^(?i:approx)\w*$'),
     )
 
-    async def runtimes(self, id):
+    async def _runtimes(self, id):
         runtimes = {}
         if id:
             soup = await self._get_soup(f'title/{id}/technical')
@@ -319,7 +321,11 @@ class ImdbApi(WebDbApiBase):
 
     def _normalize_title(self, title):
         """Return casefolded `title` without punctuation and deduplicated whitespace"""
-        return ' '.join(title.translate(self._normalize_title_translation).casefold().split())
+        title = unidecode.unidecode(title)
+        title = title.translate(self._normalize_title_translation)
+        title = title.casefold()
+        title = ' '.join(title.split())
+        return title
 
     _ignored_akas_keys = (
         re.compile(r'\(TV title\)$'),
